#-- encoding: UTF-8

#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2017 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See doc/COPYRIGHT.rdoc for more details.
#++

module OpenProject::TextFormatting
  module Filters
    class PatternMatcherFilter < HTML::Pipeline::Filter
      # Skip text nodes that are within preformatted blocks
      PREFORMATTED_BLOCKS = %w(pre code).to_set

      def self.matchers
        [
          OpenProject::TextFormatting::Matchers::ResourceLinksMatcher,
          OpenProject::TextFormatting::Matchers::WikiLinksMatcher
        ]
      end

      def call
        doc.search('.//text()').each do |node|
          next if has_ancestor?(node, PREFORMATTED_BLOCKS)

          self.class.matchers.each do |matcher|
            matcher.call(node, doc: doc, context: context)
          end
        end

        doc
      end
    end
  end
end
